{% extends "layout.html" %}
{% block head %}
  {{ super() }}
  <script src="{{ url_for('static', filename='js/scripts/login.js') }}" type="text/javascript" charset="utf-8"></script>
{% endblock %}
{% block body %}
<form method="POST" action="">
  <h2>Login to Slapos Web Runner</h2>
  <div class="login-content"><br/>
    <div class="login-element login-label"><label for="clogin">Your login&nbsp; : </label></div>
    <div class="login-element"><input type="text" class="login-input" name="clogin" id="clogin" value="Enter login..." /></div><br/><br/>
    <div class="clear"></div>
    <div class="login-element login-label"><label for="cpwd">Password : </label></div>
    <div class="login-element"><input type="password" class="idleField login-input" name="cpwd" id="cpwd" value="******" /></div>
    <div class="clear"></div>
    <!--<br/><a href="#" id="information" rel="tooltip">do you need help?</a><br/>-->
  </div>
  <div style="text-align:center; margin-top:7px;">
    <input type="reset" class="button" value="reset" />
    <input type="submit" class="button" id="login" value="login" />
  </div>
</form>
<!--
<div id="tooltip-information" style="display:none">
<p style="font-size:12px;">
If it is your first connexion, use default parameters:<br/>
login: <strong>root</strong>, password: <strong>insecure</strong> and set your
one<br/> parameters at <strong>home->Your Account</strong>.
</p>
</div>-->
{% endblock %}