UL.jqueryFileTree {
	font-family: Verdana, sans-serif;
	font-size: 12px;
	line-height: 18px;
	padding: 0px;
	margin: 0px;
}

UL.jqueryFileTree LI {
	list-style: none;
	padding: 0px;
	padding-left: 20px;
	margin: 0px;
	white-space: nowrap;
}

UL.jqueryFileTree A {
	color: #333;
	text-decoration: none;
}

UL.jqueryFileTree A:hover {
	background: #BDF;
}
.gs_dir_list{display: none;}

.gs_dir_content {
 height: 100%;
 width: 100%;
}

.gs_dir_content_menu {
 padding: 0 0 0 5px;
 margin-bottom: 5px;
 border: 1px solid #BBBBBB;
 background-color: white;
}

.gs_dir_content_button {
    background: url("data:image/gif;base64,R0lGODlhKAAVAJEAAPn5+e7u7uPj4////yH5BAEAAAMALAAAAAAoABUAAAIxhI+py+0Po5y02ouz3rz7FoTiSJbmiabqGgruC8fyTNf2jef6zvf+DwwKh8Si8SgsAAA7") repeat-x scroll 0 0 transparent;
    border: 1px outset #BBBBBB;
    cursor: pointer;
    margin-right: 7px;
    font-weight: bold;
    padding: 1px;
    background-position: 0, 50;
    text-decoration: none;
    color: black;
}

.gs_dir_content_button:hover {
	color: #EF7929;
	border: 1px inset #BBBBBB;
}

.gs_dir_content_files {
    min-height: 250px;
    max-height: 400px;
    overflow: auto;
    background-color: white;
    border: 1px solid #BBBBBB;
}

#curDir {
 height: 30px;
 line-height: 30px;
}
#gsClipBoard{display: none;}

.directory_info {
 padding-left: 20px;
}

.directory_info a {
 text-decoration: none;
}

#rootLink {
  display: none;
}

.loadingDiv {
  width: 100%;
  height: 75px;
  margin-top: 50px;
  background: url(images/loading.gif) center center no-repeat;
}

.demoto {
	width: 100%;
	height: 100%;
}

.dir_index {
  cursor: pointer;
}

.toggleplus {
	background: url('images/toggle_plus.png') left top no-repeat;
}
.toggleminus {
	background: url('images/toggle_minus.png') left top no-repeat;
}

.dirs_files_table {
 width: 100%;
 font-size: 12px;
}
.dirs_files_table td{padding: 0;border:none}
.gsItem {padding: 4px; padding-left: 25px;}
.file_ext_name {
  font-weight: normal;
  color: #FF6600;
}

.gsHeadTable {
 width :100%;
}
.dirs_files_table tr:hover {background: #BDF;}

.gsHeadText {
    color:#666;
    height:34px;
    font-weight: bold;
}
.gs_head{padding: 10px 5px 10px 0;}
.gs_title{padding-left: 10px; margin:5px 5px 0 0; background: #e4e4e4;}
.rowSelected {
	color: #FFF;
	background-color: #c2c2c2;
}

.rowSelected a {
	color: #FFF;
}

.gs_delimiter {
    height: 16px;
    line-height: 16px;
}
/* Core Styles */
.directory {
 background-image:url(images/directory.png);
 background-repeat:no-repeat;
}

.directoryMeny a { background: url(images/directory.png) left top no-repeat; padding-left: 20px}
/*.jqueryFileTree LI.expanded { background: url(images/folder_open.png) 20px top no-repeat; }*/
.file {
  background-image:url(images/file.png);
  background-repeat:no-repeat;
  cursor:pointer;
  }
.jqueryFileTree LI.wait { background: url(images/spinner.gif) left top no-repeat; }

/* File Extensions*/
.ext_3gp {
   background-image: url(images/film.png) left top no-repeat;
}
.ext_afp {
	background-image: url(images/code.png);  }
.ext_afpa {
	background-image: url(images/code.png);  }
.ext_asp {
	background-image: url(images/code.png);  }
.ext_aspx {
	background-image: url(images/code.png);  }
.ext_avi {
	background-image: url(images/film.png);  }
.ext_bat {
	background-image: url(images/application.png);  }
.ext_bmp {
	background-image: url(images/picture.png); }
.ext_c {
	background-image: url(images/code.png);  }
.ext_cfm {
	background-image: url(images/code.png); }
.ext_cgi {
	 background-image: url(images/code.png);  }
.ext_com {
	background-image: url(images/application.png);  }
.ext_cpp {
	background-image: url(images/code.png);  }
.ext_css {
	background-image: url(images/css.png);  }
.ext_doc {
	background-image: url(images/doc.png);  }
.ext_exe {
	background-image: url(images/application.png);  }
.ext_gif {
   background-image: url(images/picture.png);
}
.ext_fla {
	background-image: url(images/flash.png); }
.ext_h {
	background-image: url(images/code.png);  }
.ext_htm {
	background-image: url(images/html.png);  }
.ext_html {
	 background-image: url(images/html.png); }
.ext_jar {
	background-image: url(images/java.png); }
.ext_jpg {
	background-image: url(images/picture.png);
}
.ext_jpeg {
	background-image: url(images/picture.png);  }
.ext_js {
	background-image: url(images/script.png) ; }
.ext_lasso {
	background-image: url(images/code.png) ; }
.ext_log {
	background-image: url(images/txt.png) ; }
.ext_m4p {
	background-image: url(images/music.png);  }
.ext_mov {
	background-image: url(images/film.png);  }
.ext_mp3 {
	 background-image: url(images/music.png); }
.ext_mp4 {
	background-image: url(images/film.png);  }
.ext_mpg {
	background-image: url(images/film.png);  }
.ext_mpeg {
	 background-image: url(images/film.png);  }
.ext_ogg {
	background-image: url(images/music.png);  }
.ext_pcx {
	background-image: url(images/picture.png);  }
.ext_pdf {
	 background-image: url(images/pdf.png);  }
.ext_php {
	background-image: url(images/php.png); }
.ext_png {
	background-image: url(images/picture.png);  }
.ext_ppt {
	background-image: url(images/ppt.png);  }
.ext_psd {
	background-image: url(images/psd.png);  }
.ext_pl {
	background-image: url(images/script.png);;  }
.ext_py {
	background-image: url(images/script.png) ;}
.ext_rb {
	background-image: url(images/ruby.png); }
.ext_rbx {
	background-image: url(images/ruby.png);  }
.ext_rhtml {
	 background-image: url(images/ruby.png);  }
.ext_rpm {
	background-image: url(images/linux.png);  }
.ext_ruby {
	background-image: url(images/ruby.png);  }
.ext_sql {
	background-image: url(images/db.png); }
.ext_swf {
	background-image: url(images/flash.png);  }
.ext_tif {
	background-image: url(images/picture.png); }
.ext_tiff {
	background-image: url(images/picture.png); }
.ext_txt {
	background-image: url(images/txt.png) ; }
.ext_vb {
	background-image: url(images/code.png) ; }
.ext_wav {
	background-image: url(images/music.png) ; }
.ext_wmv {
	background-image: url(images/film.png); }
.ext_xls {
	background-image: url(images/xls.png) ; }
.ext_xml {
	background-image: url(images/code.png) ; }
.ext_zip {
	background-image: url(images/zip.png) ; }
.ext_cfg {
  background-image: url(images/cfg.png) ; }


/* Generic context menu styles */
.contextMenu {
	position: absolute;
	width: 160px;
	z-index: 99999;
	border: solid 1px #CCC;
	background: #EEE;
	padding: 0px;
	margin: 0px;
	display: none;
}

.contextMenu LI {
	list-style: none;
	padding: 0px;
	margin: 0px;
}

.contextMenu A {
	color: #333;
	text-decoration: none;
	display: block;
	line-height: 20px;
	height: 20px;
	background-position: 6px center;
	background-repeat: no-repeat;
	outline: none;
	padding: 1px 5px;
	padding-left: 28px;
}

.contextMenu LI.hover A {
	color: #FFF;
	background-color: #3399FF;
}

.contextMenu LI.disabled A {
	color: #AAA;
	cursor: default;
}

.contextMenu LI.hover.disabled A {
	background-color: transparent;
}

.contextMenu LI.separator {
	border-top: solid 1px #CCC;
}

/*
	Adding Icons

	You can add icons to the context menu by adding
	classes to the respective LI element(s)
*/

.contextMenu LI.edit A { background-image: url(images/page_white_edit.png); }
.contextMenu LI.cut A { background-image: url(images/cut.png); }
.contextMenu LI.copy A { background-image: url(images/page_white_copy.png); }
.contextMenu LI.paste A { background-image: url(images/page_white_paste.png); }
.contextMenu LI.delete A { background-image: url(images/cross.png); }
.contextMenu LI.quit A { background-image: url(images/door.png); }
.contextMenu LI.directorymenu A { background-image: url(images/folder_open.png); }
.contextMenu LI.rename A { background-image: url(images/mfile.png); }
.contextMenu LI.download A { background-image: url(images/disk.png); }
.contextMenu LI.notepad A { background-image: url(images/page_white_edit.png); }
.contextMenu LI.picture A { background-image: url(images/mpicture.png); }
.contextMenu LI.newfile A { background-image: url(images/new_file.png); }
.contextMenu LI.newdir A { background-image: url(images/new_dir.png); }
.contextMenu LI.uploadfolder A { background-image: url(images/upload_folder.png); }
.contextMenu LI.selection A { background-image: url(images/selection-select.png); }
.contextMenu LI.zip A { background-image: url(images/mzip.png); }